import 'package:fis_common/json_convert.dart';

import '../client_base.dart';
import 'platform.m.dart';

/// 平台服务
class PlatformService extends JsonRpcClientBase {
  PlatformService(
    String host, {
    String serviceName = "IPlatformService",
    Map<String, String>? headers,
    int? timeout,
  }) : super(
          host,
          serviceName,
          headers: headers,
          timeout: timeout,
        ) {
    /// 注册响应实体反序列化处理器
    FJsonConvert.setDecoder((map) => LoadVidResult.fromJson(map));
    FJsonConvert.setDecoder((map) => GetVidFrameResult.fromJson(map));
  }

  /// 加载主题
  Future<bool> loadTheme(String name) async {
    var rpcRst = await call("LoadTheme", name);
    return rpcRst;
  }

  /// 保存配置
  ///
  /// [jsonText] 配置json文本
  Future<bool> saveConfig(String jsonText) async {
    var rpcRst = await call("SaveConfig", jsonText);
    return rpcRst;
  }

  /// 获取配置
  Future<String?> getConfig() async {
    var rpcRst = await call("GetConfig");
    return rpcRst;
  }

  /// 保存文本文件
  ///
  /// [name] 文件名
  ///
  /// [text] 文本
  Future<bool> saveText(String name, String text) async {
    var rpcRst = await call("SaveText", [name, text]);
    return rpcRst;
  }

  /// 获取文件文本
  ///
  /// [name] 文件名
  Future<String?> getText(String name) async {
    var rpcRst = await call("GetText", name);
    return rpcRst;
  }

  /// 加载Vid文件
  ///
  /// [url] Vid文件链接
  Future<LoadVidResult> loadVid(String url) async {
    var rpcRst = await call("LoadVid", url);
    var result = LoadVidResult.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  /// 获取Vid单针帧图
  Future<GetVidFrameResult> getVidFrame(GetVidFrameRequest request) async {
    var rpcRst = await call("GetVidFrame", request);
    var result = GetVidFrameResult.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  /// 释放Vid缓存资源
  void releaseVid() => notify("ReleaseVid");
}
