import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'report.m.dart';

import 'patient.m.dart';


class ReportService extends JsonRpcClientBase {
	ReportService(
		String host, {
		String serviceName = "IReportService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => PageResult<ReportTemplateDTO>.fromJson(map));
		FJsonConvert.setDecoder((map) => ReportTemplateDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => ReportDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => ReportLabelDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => ShareDTO.fromJson(map));
	}

	Future<PageResult<ReportTemplateDTO>> findReportTemplateByPageAsync(FindReportTemplateByPageRequest request) async {
		var rpcRst = await call("FindReportTemplateByPageAsync", request);
		var result = PageResult<ReportTemplateDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<ReportTemplateDTO> findReportTemplateByCodeAsync(FindReportTemplateByCodeRequest request) async {
		var rpcRst = await call("FindReportTemplateByCodeAsync", request);
		var result = ReportTemplateDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<ReportDTO> findReportByCodeAsync(FindReportByCodeRequest request) async {
		var rpcRst = await call("FindReportByCodeAsync", request);
		var result = ReportDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<ReportDTO>> findExamReportsAsync(FindExamReportsRequest request) async {
		var rpcRst = await call("FindExamReportsAsync", request);
		var result = (rpcRst as List).map((e)=>ReportDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<List<ReportLabelDTO>> findReportLabelsAsync(FindReportLabelsRequest request) async {
		var rpcRst = await call("FindReportLabelsAsync", request);
		var result = (rpcRst as List).map((e)=>ReportLabelDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<ShareDTO> findShareContentAsync(FindShareContentRequest request) async {
		var rpcRst = await call("FindShareContentAsync", request);
		var result = ShareDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> addReportAsync(AddReportRequest request) async {
		var rpcRst = await call("AddReportAsync", request);
		return rpcRst;
	}

	Future<bool> modifyReportAsync(ModifyReportRequest request) async {
		var rpcRst = await call("ModifyReportAsync", request);
		return rpcRst;
	}

	Future<bool> modifyReportLabelsAsync(ModifyReportLabelsRequest request) async {
		var rpcRst = await call("ModifyReportLabelsAsync", request);
		return rpcRst;
	}

	Future<bool> shareReportBySmsAsync(ShareReportBySmsRequest request) async {
		var rpcRst = await call("ShareReportBySmsAsync", request);
		return rpcRst;
	}

	Future<bool> shareReportByEmailAsync(ShareReportByEmailRequest request) async {
		var rpcRst = await call("ShareReportByEmailAsync", request);
		return rpcRst;
	}

}

